#[macro_use]
extern crate diesel;
extern crate dotenv;

pub mod db_connection;
pub mod errors;
pub mod handlers;
pub mod models;
pub mod schema;
pub mod utils;

use actix_cors::Cors;
use actix_identity::{CookieIdentityPolicy, IdentityService};
use actix_web::{http::header, middleware::Logger, App, HttpServer};
use chrono::Duration;
use db_connection::get_pool;
use dotenv::dotenv;
use handlers::authentication;
use std::env;

#[actix_rt::main]
async fn main() -> std::io::Result<()> {
    std::env::set_var("RUST_LOG", "actix_web=debug,diesel=debug");
    env_logger::init();
    dotenv().ok();

    let bind = "localhost:8080";
    println!("Starting server at: http://{}", &bind);

    HttpServer::new(move || {
        App::new()
            .wrap(Logger::default())
            .wrap(
                Cors::new()
                    .allowed_origin(env::var("ALLOWED_ORIGIN").expect("ALLOWED_ORIGIN").as_str())
                    .allowed_methods(vec!["GET", "POST", "PUT", "DELETE"])
                    .allowed_headers(vec![
                        header::AUTHORIZATION,
                        header::CONTENT_TYPE,
                        header::ACCEPT,
                    ])
                    .max_age(3600)
                    .finish(),
            )
            .wrap(IdentityService::new(
                CookieIdentityPolicy::new(env::var("SECRET").expect("SECRET").as_bytes())
                    .domain(env::var("DOMAIN").expect("DOMAIN"))
                    .name("thesis")
                    .path("/")
                    .max_age(Duration::days(1).num_seconds())
                    .secure(false),
            ))
            .data(get_pool())
            .service(authentication::register)
            .service(authentication::login)
    })
    .bind(&bind)?
    .run()
    .await
}
